
# -*- coding: utf8 -*-
# SPDX-License-Identifier: GPL-3.0-or-later
"""
Plotting functions for seiscat.

:copyright:
    2022-2023 Claudio Satriano <satriano@ipgp.fr>
:license:
    GNU General Public License v3.0 or later
    (https://www.gnu.org/licenses/gpl-3.0-standalone.html)
"""
import sqlite3
import numpy as np
import matplotlib.pyplot as plt
from obspy import UTCDateTime
from .utils import err_exit


def _get_map_extent_for_suffix(config, suffix=None):
    """
    Get the map extent for a suffix.

    :param config: config object
    :param suffix: suffix to be added to the config keys
    :returns: lon_min, lon_max, lat_min, lat_max
    """
    suffix = '' if suffix is None else suffix
    lat_min = config.get(f'lat_min{suffix}', None)
    lat_max = config.get(f'lat_max{suffix}', None)
    lon_min = config.get(f'lon_min{suffix}', None)
    lon_max = config.get(f'lon_max{suffix}', None)
    lat0 = config.get(f'lat0{suffix}', None)
    lon0 = config.get(f'lon0{suffix}', None)
    radius_max = config.get(f'radius_max{suffix}', None)
    if None not in (lat0, lon0, radius_max):
        lat_min = lat0 - radius_max * np.sqrt(2)
        lat_max = lat0 + radius_max * np.sqrt(2)
        lon_min = lon0 - radius_max * np.sqrt(2)
        lon_max = lon0 + radius_max * np.sqrt(2)
        return lon_min, lon_max, lat_min, lat_max
    if None not in (lat_min, lat_max, lon_min, lon_max):
        return lon_min, lon_max, lat_min, lat_max
    return None


def _get_map_extent(config):
    """
    Get the map extent from the config file.

    :param config: config object
    :returns: lon_min, lon_max, lat_min, lat_max
    """
    ret = _get_map_extent_for_suffix(config)
    if ret is None:
        lon_min = -179
        lon_max = 180
        lat_min = -75
        lat_max = 80
    else:
        lon_min, lon_max, lat_min, lat_max = ret
    # see if there are additional limits in the config file
    n = 1
    while True:
        ret = _get_map_extent_for_suffix(config, suffix=f'_{n}')
        if ret is None:
            break
        lon_min_, lon_max_, lat_min_, lat_max_ = ret
        lon_min = min(lon_min, lon_min_)
        lon_max = max(lon_max, lon_max_)
        lat_min = min(lat_min, lat_min_)
        lat_max = max(lat_max, lat_max_)
        n += 1
    return lon_min, lon_max, lat_min, lat_max


def _get_tile_scale(extent):
    """
    Get the tile scale for a given extent.

    :param extent: tuple (lon_min, lon_max, lat_min, lat_max)
    :returns: tile scale
    """
    from cartopy.feature import AdaptiveScaler
    tile_autoscaler = AdaptiveScaler(
        default_scale=4,
        limits=(
            (4, 180), (5, 90), (6, 45), (7, 25), (8, 15), (9, 5),
            (10, 2), (11, 1),
        )
    )
    tile_scale = tile_autoscaler.scale_from_extent(extent)
    # print(f'tile_scale: {tile_scale}')
    return int(tile_scale)


def _read_event_db(config):
    """
    Read event database. Return a list of events.

    :param config: config object
    :returns: list of events, each event is a dictionary
    """
    db_file = config.get('db_file', None)
    if db_file is None:
        err_exit('db_file not set in config file')
    try:
        open(db_file, 'r')
    except FileNotFoundError:
        err_exit(f'Database file "{db_file}" not found.')
    conn = sqlite3.connect(db_file)
    c = conn.cursor()
    # read field names
    c.execute('PRAGMA table_info(events)')
    fields = c.fetchall()
    # read events
    c.execute('SELECT * FROM events')
    events = c.fetchall()
    conn.close()
    # create a list of dictionaries
    events_list = []
    for event in events:
        event_dict = {field[1]: event[i] for i, field in enumerate(fields)}
        event_dict['time'] = UTCDateTime(event_dict['time'])
        events_list.append(event_dict)
    return events_list


def _plot_events(events, scale, ax):
    """
    Plot events on a map.

    :param events: list of events, each event is a dictionary
    :param scale: scale for event markers
    :param ax: matplotlib axes object
    """
    import cartopy.crs as ccrs
    marker_scale = scale / 10. * 2
    ev_attributes = [
        (e['evid'], e['time'], e['lon'], e['lat'], e['dep'],
         e['mag'], np.exp(e['mag']) * marker_scale)
        for e in events
    ]
    markers = []
    for evid, time, lon, lat, dep, mag, size in ev_attributes:
        marker_label = (
            f'{evid} M{mag:.1f} {dep:.1f} km\n'
            f'{time.strftime("%Y-%m-%d %H:%M:%S")}')
        marker = ax.scatter(
            lon, lat,
            s=size,
            facecolor='red', edgecolor='black',
            label=marker_label,
            zorder=10,
            transform=ccrs.PlateCarree(),
        )
        markers.append(marker)
    # Empty annotation that will be updated interactively
    annot = ax.annotate(
        '', xy=(0, 0), xytext=(5, 5),
        textcoords='offset points',
        bbox=dict(boxstyle='round', fc='w'),
        zorder=20
    )
    annot.set_visible(False)
    fig = ax.get_figure()

    def hover(event):
        vis = annot.get_visible()
        if vis:
            annot.set_visible(False)
            fig.canvas.draw_idle()
        if event.inaxes != ax:
            return
        for marker in markers:
            cont, _ = marker.contains(event)
            if cont:
                marker.set_linewidth(3)
                annot.xy = (event.xdata, event.ydata)
                annot.set_text(marker.get_label())
                annot.get_bbox_patch().set_facecolor('white')
                annot.get_bbox_patch().set_alpha(0.8)
                annot.set_visible(True)
            else:
                marker.set_linewidth(1)
        fig.canvas.draw_idle()
    fig.canvas.mpl_connect('motion_notify_event', hover)


def plot_catalog_map(config):
    """
    Plot the catalog map.

    :param config: config object
    """
    # lazy import cartopy, since it's not an install requirement
    try:
        import cartopy.crs as ccrs
        import cartopy.io.img_tiles as cimgt
    except ImportError:
        err_exit(
            'Cartopy is not installed. '
            'Please install it to plot the catalog map.\n'
            'See https://scitools.org.uk/cartopy/docs/latest/installing.html'
        )
    fig = plt.figure(figsize=(10, 10))
    stamen_terrain = cimgt.Stamen('terrain-background')
    ax = fig.add_subplot(1, 1, 1, projection=stamen_terrain.crs)
    extent = _get_map_extent(config)
    ax.set_extent(extent)
    tile_scale = _get_tile_scale(extent)
    ax.add_image(stamen_terrain, tile_scale)
    ax.coastlines(resolution='10m', edgecolor='black', linewidth=1)
    ax.add_feature(
        ccrs.cartopy.feature.BORDERS, edgecolor='black', linewidth=1)
    g_kwargs = dict(draw_labels=True, dms=True, x_inline=False, y_inline=False)
    ax.gridlines(**g_kwargs)

    def redraw_gridlines(ax):
        ax.gridlines(**g_kwargs)
    ax.callbacks.connect('xlim_changed', lambda ax: redraw_gridlines(ax))
    events = _read_event_db(config)
    _plot_events(events, config['args'].scale, ax)
    nevents = len(events)
    tmin = min(event['time'] for event in events)
    tmax = max(event['time'] for event in events)
    tmin = tmin.strftime('%Y-%m-%dT%H:%M:%S')
    tmax = tmax.strftime('%Y-%m-%dT%H:%M:%S')
    mag_min = min(event['mag'] for event in events)
    mag_max = max(event['mag'] for event in events)
    ax.set_title(
        f'{nevents} events from {tmin} to {tmax}\n'
        f'Magnitude range: {mag_min:.1f} - {mag_max:.1f}'
    )
    plt.show()



#!/usr/bin/env python
# -*- coding: utf8 -*-
# SPDX-License-Identifier: GPL-3.0-or-later
"""
Main script for seiscat.

:copyright:
    2022-2023 Claudio Satriano <satriano@ipgp.fr>
:license:
    GNU General Public License v3.0 or later
    (https://www.gnu.org/licenses/gpl-3.0-standalone.html)
"""
import sys
import argparse
import argcomplete
from .._version import get_versions
# NOTE: most modules are lazy-imported to speed up startup time


def parse_arguments():
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(
        description='Keep a local seismic catalog.')
    subparser = parser.add_subparsers(dest='action')
    subparser.add_parser('initdb', help='initialize database')
    subparser.add_parser('updatedb', help='update database')
    subparser.add_parser('print', help='print catalog')
    plot_parser = subparser.add_parser('plot', help='plot catalog map')
    plot_parser.add_argument(
        '-s',
        '--scale',
        type=float,
        default=10,
        help='scale factor for marker size (default: %(default)s)'
    )
    subparser.add_parser('sampleconfig', help='write sample config file')
    parser.add_argument(
        '-c',
        '--configfile',
        type=str,
        default='seiscat.conf',
        help='config file for data sources and processing params'
    )
    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version=f"%(prog)s {get_versions()['version']}",
    )
    argcomplete.autocomplete(parser)
    args = parser.parse_args()
    if args.action is None:
        parser.print_help()
        sys.exit(0)
    return args


def download_and_store(config, initdb):
    """
    Download events and store them in the database.

    :param config: config object
    :param initdb: if True, create new database file
    """
    from ..db import check_db_exists, write_catalog_to_db
    from ..fdsnws import open_fdsn_connection, query_events
    check_db_exists(config, initdb)
    client = open_fdsn_connection(config)
    cat = query_events(client, config, first_query=initdb)
    write_catalog_to_db(cat, config, initdb)


def run():
    """Run seiscat."""
    args = parse_arguments()
    from ..utils import parse_configspec, read_config,  write_sample_config
    configspec = parse_configspec()
    if args.action == 'sampleconfig':
        write_sample_config(configspec, 'seiscat')
        sys.exit(0)
    config = read_config(args.configfile, configspec)
    config['args'] = args
    if args.action == 'initdb':
        download_and_store(config, initdb=True)
    elif args.action == 'updatedb':
        download_and_store(config, initdb=False)
    elif args.action == 'print':
        from ..print import print_catalog
        print_catalog(config)
    elif args.action == 'plot':
        from ..plot import plot_catalog_map
        plot_catalog_map(config)


def main():
    """Main function. Catch KeyboardInterrupt."""
    try:
        run()
    except KeyboardInterrupt:
        sys.exit(1)



# -*- coding: utf8 -*-
# SPDX-License-Identifier: GPL-3.0-or-later
"""
Printing functions for seiscat.

:copyright:
    2022-2023 Claudio Satriano <satriano@ipgp.fr>
:license:
    GNU General Public License v3.0 or later
    (https://www.gnu.org/licenses/gpl-3.0-standalone.html)
"""
import sqlite3
from .utils import err_exit


def print_catalog(config):
    """
    Pretty-print the catalog as a table.

    :param config: config object
    """
    db_file = config.get('db_file', None)
    if db_file is None:
        err_exit('db_file not set in config file')
    try:
        open(db_file, 'r')
    except FileNotFoundError:
        err_exit(f'Database file "{db_file}" not found.')
    conn = sqlite3.connect(db_file)
    c = conn.cursor()
    # get fields
    c.execute('PRAGMA table_info(events)')
    fields = c.fetchall()
    c.execute('SELECT * FROM events')
    rows = c.fetchall()
    if len(rows) == 0:
        print('No events in catalog')
        return
    # get max length of each field
    max_len = [len(f[1]) for f in fields]
    for row in rows:
        for i, val in enumerate(row):
            max_len[i] = max(max_len[i], len(str(val)))
    # print header
    for i, f in enumerate(fields):
        print(f'{f[1]:{max_len[i]}}', end=' ')
    print()
    # print rows
    for row in rows:
        for i, val in enumerate(row):
            val = 'None' if val is None else val
            print(f'{val:{max_len[i]}}', end=' ')
        print()
    conn.close()



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: GPL-3.0-or-later
"""
Utility functions for seiscat.

:copyright:
    2022-2023 Claudio Satriano <satriano@ipgp.fr>
:license:
    GNU General Public License v3.0 or later
    (https://www.gnu.org/licenses/gpl-3.0-standalone.html)
"""
import os
import sys
from .configobj import ConfigObj
from .configobj.validate import Validator


def err_exit(msg):
    """
    Print an error message and exit.

    :param msg: error message
    """
    msg = str(msg)
    sys.stderr.write(msg + '\n')
    sys.exit(1)


def parse_configspec():
    """
    Parse the configspec file.

    :returns: configspec object
    """
    curdir = os.path.dirname(__file__)
    configspec_file = os.path.join(curdir, 'conf', 'configspec.conf')
    return read_config(configspec_file)


def write_ok(filepath):
    """
    Check if it is ok to write to a file.

    :param filepath: path to the file
    :returns: True if it is ok to write to the file, False otherwise
    """
    if os.path.exists(filepath):
        ans = input(
            f'"{filepath}" already exists. Do you want to overwrite it? [y/N] '
        )
        return ans in ['y', 'Y']
    return True


def write_sample_config(configspec, progname):
    """
    Write a sample config file.

    :param configspec: configspec object
    :param progname: program name
    """
    c = ConfigObj(configspec=configspec, default_encoding='utf8')
    val = Validator()
    c.validate(val)
    c.defaults = []
    c.initial_comment = configspec.initial_comment
    c.comments = configspec.comments
    c.final_comment = configspec.final_comment
    configfile = f'{progname}.conf'
    if write_ok(configfile):
        with open(configfile, 'wb') as fp:
            c.write(fp)
        print(f'Sample config file written to: "{configfile}"')


def read_config(config_file, configspec=None):
    """
    Read a config file.

    :param config_file: path to the config file
    :param configspec: configspec object
    :returns: config object
    """
    kwargs = dict(
        configspec=configspec, file_error=True, default_encoding='utf8')
    if configspec is None:
        kwargs.update(
            dict(interpolation=False, list_values=False, _inspec=True))
    try:
        config_obj = ConfigObj(config_file, **kwargs)
    except IOError as err:
        err_exit(err)
    except Exception as err:
        msg = f'Unable to read "{config_file}": {err}'
        err_exit(msg)
    for k, v in config_obj.items():
        if v == 'None':
            config_obj[k] = None
    if configspec is not None:
        _validate_config(config_obj)
    return config_obj


def _validate_config(config_obj):
    """
    Validate the config object.

    :param config_obj: config object
    """
    configspec = config_obj.configspec
    config_obj_keys = list(config_obj.keys())
    configspec_keys = list(configspec.keys())
    # extend configspec with keys ending with _n, if present in config_obj
    n = 1
    while True:
        matching_keys = [k for k in config_obj_keys if k.endswith(f'_{n}')]
        if not matching_keys:
            break
        for k in configspec_keys:
            configspec[f'{k}_{n}'] = configspec[k]
        n += 1
    config_obj.configspec = configspec
    val = Validator()
    test = config_obj.validate(val)
    if isinstance(test, dict):
        for entry in test:
            if not test[entry]:
                sys.stderr.write(
                    f'Invalid value for "{entry}": "{config_obj[entry]}"\n')
        sys.exit(1)
    if not test:
        err_exit('No configuration value present!')



# -*- coding: utf8 -*-
# SPDX-License-Identifier: GPL-3.0-or-later
"""
FDSN webservices functions for seiscat.

:copyright:
    2022-2023 Claudio Satriano <satriano@ipgp.fr>
:license:
    GNU General Public License v3.0 or later
    (https://www.gnu.org/licenses/gpl-3.0-standalone.html)
"""
from datetime import timedelta
from obspy import UTCDateTime
from obspy import Catalog
from obspy.clients.fdsn import Client
from obspy.clients.fdsn.header import FDSNNoDataException
from .utils import err_exit


def open_fdsn_connection(config):
    """
    Open FDSN connection. Return a FDSN client object.

    :param config: config object
    :returns: FDSN client object
    """
    return Client(config['fdsn_event_url'])


def _to_utc_datetime(time):
    """
    Convert time to UTCDateTime object.

    :param time: time in string format
    :returns: UTCDateTime object or None
    """
    return None if time is None else UTCDateTime(time)


def _parse_time_interval(time_interval):
    """
    Parse time interval string.

    :param time_interval: time interval in string format
    :returns: timedelta object or None
    """
    if time_interval is None:
        return None
    parts = time_interval.split()
    value = int(parts[0])
    unit = parts[1]
    if unit.endswith('s'):
        unit = unit[:-1]
    if unit == 'day':
        return timedelta(days=value)
    elif unit == 'hour':
        return timedelta(hours=value)
    elif unit == 'minute':
        return timedelta(minutes=value)
    elif unit == 'second':
        return timedelta(seconds=value)
    else:
        raise ValueError(f'Invalid time unit: {unit}')


class InvalidQuery(Exception):
    """Invalid query exception."""
    pass


class QueryArgs(object):
    """Build query arguments for FDSN client."""
    def __init__(self, config, suffix, first_query):
        """
        Initialize query arguments.

        :param config: config object
        :param suffix: suffix to be added to the config keys
        :param first_query: True if this is the first query
        """
        query_keys = [
            'start_time', 'end_time', 'recheck_period',
            'lat_min', 'lat_max', 'lon_min', 'lon_max',
            'lat0', 'lon0', 'radius_min', 'radius_max',
            'depth_min', 'depth_max',
            'mag_min', 'mag_max',
            'event_type', 'event_type_exclude'
        ]
        if suffix:
            query_keys = [k + suffix for k in query_keys]
        try:
            if all(config[k] is None for k in query_keys):
                raise InvalidQuery(
                    'All query parameters are None. Please set at least one.')
        except KeyError as e:
            raise InvalidQuery('Not all query parameters are set.') from e
        self.starttime = _to_utc_datetime(config[f'start_time{suffix}'])
        self.endtime = _to_utc_datetime(config[f'end_time{suffix}'])
        recheck_period = _parse_time_interval(
            config[f'recheck_period{suffix}'])
        if not first_query and self.endtime is None and recheck_period:
            self.starttime = max(
                self.starttime, UTCDateTime() - recheck_period)
        self.minlatitude = config[f'lat_min{suffix}']
        self.maxlatitude = config[f'lat_max{suffix}']
        self.minlongitude = config[f'lon_min{suffix}']
        self.maxlongitude = config[f'lon_max{suffix}']
        self.latitude = config[f'lat0{suffix}']
        self.longitude = config[f'lon0{suffix}']
        self.minradius = config[f'radius_min{suffix}']
        self.maxradius = config[f'radius_max{suffix}']
        self.mindepth = config[f'depth_min{suffix}']
        self.maxdepth = config[f'depth_max{suffix}']
        self.minmagnitude = config[f'mag_min{suffix}']
        self.maxmagnitude = config[f'mag_max{suffix}']

    def get_query(self):
        """
        Return query arguments as a dictionary.

        :returns: dictionary of query arguments
        """
        return {
            'starttime': self.starttime, 'endtime': self.endtime,
            'minlatitude': self.minlatitude, 'maxlatitude': self.maxlatitude,
            'minlongitude': self.minlongitude,
            'maxlongitude': self.maxlongitude,
            'latitude': self.latitude, 'longitude': self.longitude,
            'minradius': self.minradius, 'maxradius': self.maxradius,
            'mindepth': self.mindepth, 'maxdepth': self.maxdepth,
            'minmagnitude': self.minmagnitude,
            'maxmagnitude': self.maxmagnitude,
        }


def _query_box_or_circle(client, config, suffix=None, first_query=True):
    """
    Query events from FDSN client based on box or circle criteria in config.

    :param client: FDSN client object
    :param config: config object
    :param suffix: suffix to be added to the config keys
    :param first_query: True if this is the first query
    :returns: obspy Catalog object
    """
    suffix = '' if suffix is None else suffix
    query_args = QueryArgs(config, suffix, first_query)
    kwargs = query_args.get_query()
    try:
        cat = client.get_events(**kwargs)
    except FDSNNoDataException:
        cat = Catalog()
    # filter in included event types
    event_type = config[f'event_type{suffix}']
    if event_type:
        cat = Catalog([
            ev for ev in cat
            if ev.event_type in event_type
        ])
    # filter out excluded event types
    event_type_exclude = config[f'event_type_exclude{suffix}']
    if event_type_exclude:
        cat = Catalog([
            ev for ev in cat
            if ev.event_type not in event_type_exclude
        ])
    return cat


def query_events(client, config, first_query=True):
    """
    Query events from FDSN client based on criteria in config.

    :param client: FDSN client object
    :param config: config object
    :param first_query: True if this is the first query
    :returns: obspy Catalog object
    """
    print(f'Querying events from FDSN server "{config["fdsn_event_url"]}"...')
    try:
        cat = _query_box_or_circle(client, config, first_query=first_query)
    except Exception as e:
        err_exit(e)
    # see if there are additional queries to be done
    n = 1
    while True:
        try:
            _cat = _query_box_or_circle(
                client, config, suffix=f'_{n}', first_query=first_query)
        except InvalidQuery:
            break
        except Exception as e:
            err_exit(e)
        cat += _cat
        n += 1
    print(f'Found {len(cat)} events.')
    return cat



# -*- coding: utf8 -*-
# SPDX-License-Identifier: GPL-3.0-or-later
"""
Database functions for seiscat.

:copyright:
    2022-2023 Claudio Satriano <satriano@ipgp.fr>
:license:
    GNU General Public License v3.0 or later
    (https://www.gnu.org/licenses/gpl-3.0-standalone.html)
"""
import os
import sqlite3
from .utils import err_exit

# Current supported DB version
# Increment this number when changing the DB schema
DB_VERSION = 1


def _check_db_version(cursor, config):
    """
    Check if database version is compatible with current version.

    :param cursor: database cursor
    :param config: config object
    """
    db_version = cursor.execute('PRAGMA user_version').fetchone()[0]
    db_file = config['db_file']
    if db_version < DB_VERSION:
        msg = (
            f'"{db_file}" has an old database version: '
            f'"{db_version}" Current supported version is "{DB_VERSION}".\n'
            'Remove or rename your old database file, '
            'so that a new one can be created.'
        )
        err_exit(msg)


def _set_db_version(cursor):
    """
    Set the database version.

    :param cursor: database cursor
    """
    cursor.execute('PRAGMA user_version = {v:d}'.format(v=DB_VERSION))


def check_db_exists(config, initdb):
    """
    Check if database file exists.

    :param config: config object
    :param initdb: if True, create new database file
    """
    db_file = config.get('db_file', None)
    if db_file is None:
        err_exit('db_file not set in config file')
    if initdb and os.path.exists(db_file):
        ans = input(
            f'"{db_file}" already exists. '
            'Do you want to overwrite it? [y/N] '
        )
        if ans not in ['y', 'Y']:
            err_exit('Database file already exists. Exiting.')
        else:
            os.rename(
                db_file, f'{db_file}.bak')
            print(
                f'Backup of "{db_file}" saved to '
                f'"{db_file}.bak"')
    if not initdb and not os.path.exists(db_file):
        err_exit(
            f'Database file "{db_file}" does not exist.\n'
            'Run "seiscat initdb" first.'
        )


def _get_evid(resource_id):
    """
    Get evid from resource_id.

    :param resource_id: resource_id string
    :returns: evid string
    """
    evid = resource_id
    if '/' in evid:
        evid = resource_id.split('/')[-1]
    if '?' in evid:
        evid = resource_id.split('?')[-1]
    if '&' in evid:
        evid = evid.split('&')[0]
    if '=' in evid:
        evid = evid.split('=')[-1]
    return evid


def write_catalog_to_db(cat, config, initdb):
    """
    Write catalog to database.

    :param cat: obspy Catalog object
    :param config: config object
    :param initdb: if True, create new database file
    """
    # open database connection
    conn = sqlite3.connect(config['db_file'])
    c = conn.cursor()
    if initdb:
        _set_db_version(c)
    else:
        _check_db_version(c, config)
    # table fields: name TYPE
    fields = [
        'evid TEXT PRIMARY KEY',
        'time TEXT',
        'lat REAL',
        'lon REAL',
        'dep REAL',
        'mag REAL',
        'mag_type TEXT',
        'event_type TEXT',
    ]
    extra_field_names = config['extra_field_names'] or []
    extra_field_types = config['extra_field_types'] or []
    fields.extend(
        f'{name} {dbtype}' for name, dbtype
        in zip(extra_field_names, extra_field_types))
    # create table if it doesn't exist
    c.execute(f'CREATE TABLE IF NOT EXISTS events ({", ".join(fields)})')
    events_written = 0
    for ev in cat:
        evid = _get_evid(str(ev.resource_id.id))
        orig = ev.preferred_origin() or ev.origins[0]
        time = str(orig.time)
        lat = orig.latitude
        lon = orig.longitude
        dep = orig.depth / 1e3  # km
        magntiude = ev.preferred_magnitude() or ev.magnitudes[0]
        mag = magntiude.mag
        mag_type = magntiude.magnitude_type
        event_type = ev.event_type
        values = [evid, time, lat, lon, dep, mag, mag_type, event_type]
        # add extra fields
        extra_field_defaults = config['extra_field_defaults'] or []
        values += extra_field_defaults
        if initdb:
            # add events to table, replace events that already exist
            c.execute(
                'INSERT OR REPLACE INTO events VALUES '
                f'({", ".join("?" * len(values))})', values)
        else:
            # add events to table, ignore events that already exist
            c.execute(
                'INSERT OR IGNORE INTO events VALUES '
                f'({", ".join("?" * len(values))})', values)
        events_written += c.rowcount
    # close database connection
    conn.commit()
    print(f'Wrote {events_written} events to database "{config["db_file"]}"')



# -*- coding: utf-8 -*-
"""setup.py: setuptools control."""
from setuptools import setup
import versioneer

with open('README.md', 'rb') as f:
    long_descr = f.read().decode('utf-8')

project_urls = {
    'Homepage': 'https://seiscat.seismicsource.org',
    'Source': 'https://github.com/SeismicSource/seiscat',
    'Documentation': 'https://seiscat.readthedocs.io'
}

setup(
    name='seiscat',
    packages=['seiscat', 'seiscat.scripts', 'seiscat.configobj'],
    include_package_data=True,
    entry_points={
        'console_scripts': ['seiscat = seiscat.scripts.seiscat:main']
    },
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description='Keep a local seismic catalog',
    long_description=long_descr,
    long_description_content_type='text/markdown',
    author='Claudio Satriano',
    author_email='satriano@ipgp.fr',
    url=project_urls['Homepage'],
    project_urls=project_urls,
    license='GNU General Public License v3 or later (GPLv3+)',
    platforms='OS Independent',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: '
            'GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Physics'],
    install_requires=['obspy>=1.1.0', 'argcomplete'],
    python_requires='>=3.7'
)
